#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include "GL/glew.h"
#include "GL/wglew.h"
#include "glm/glm.hpp"
#include "glm/ext.hpp"
#include "AL/alut.h"
#include "btBulletDynamicsCommon.h"
#include "BulletCollision/CollisionDispatch/btCollisionDispatcherMt.h"
#include "BulletDynamics/Dynamics/btDiscreteDynamicsWorldMt.h"
#include "BulletDynamics/ConstraintSolver/btSequentialImpulseConstraintSolverMt.h"

#include "Shader.h"
#include "Model.h"
#include "Camera.h"
#include "SkyBox.h"
#include "RenderTarget.h"

#include <Windows.h>

#include <QSplashScreen>
#include <QMainWindow>
#include <QTimer>
#include <QElapsedTimer>
#include <QKeyEvent>
#include <QMouseEvent>

QT_BEGIN_NAMESPACE
namespace Ui { class MainWindow; }
QT_END_NAMESPACE

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    MainWindow(QWidget *parent = nullptr);
    ~MainWindow();

    bool Init(QSplashScreen &splash);

private slots:
    bool eventFilter(QObject *obj, QEvent *event);
    void keyPressEvent(QKeyEvent *event);
    void keyReleaseEvent(QKeyEvent *event);
    void mousePressEvent(QMouseEvent *event);
    void mouseReleaseEvent(QMouseEvent *event);
    void mouseMoveEvent(QMouseEvent *event);

    void TimerTick();

private:
    Ui::MainWindow *ui;

    // opengl
    bool InitGL(QWidget *pWidget);
    void CreateShadowMapFramebuffer();

    // physics
    bool InitPhysics();
    void ExitPhysics();
    // scene
    bool InitScene();
    btRigidBody* CreateConvexMesh(glm::vec3 v3Translate, glm::vec3 v3RotateEuler, float fMass, float fRestitution, float fFriction, std::vector<Vertex> *pListVertices);
    btRigidBody* CreateConcaveMesh(glm::vec3 v3Translate, glm::vec3 v3RotateEuler, float fMass, float fRestitution, float fFriction, std::vector<Vertex> *pListVertices);
    btRigidBody* CreateAvatar(glm::vec3 v3Pos, float radius, float height);

    // OpenGL
    HWND hWnd;
    HDC hDC;
    HGLRC hRC;

    // SkyBox
    SkyBox m_SkyBox;

    // Model
    Model m_modelDraw;
    Model m_modelPhysics;

    // shader
    Shader m_shaderDraw;
    Shader m_shaderShadowMap;

    // RenderTarget
    RenderTarget m_RenderToShadowTexture;

    // SYSTEM
    QTimer m_Timer;
    Camera m_Camera;
    bool   m_bKeys[256];
    //bool m_bMouseButtonDown = false;
    QPoint m_pointElapsedMousePos;

    // FPS
    QElapsedTimer m_elapsedTimer;
    uint64_t m_nElapsedTime;
    uint64_t m_nCurrentTime;
    float dt;
    int nFPS = 0;
    float fSec = 0;

    // ShadowMapping FrameBuffer
    GLuint framebuffer = 0;
    GLuint depthBuffer = 0;

    int nShadowWidth = 2048;
    GLuint depthTexture = 0;

    // bullet physics
    btDefaultCollisionConfiguration* collisionConfiguration;
    btCollisionDispatcher* dispatcher;
    btDbvtBroadphase* broadphase;
    btConstraintSolverPoolMt* solverPool;
    btConstraintSolver* parallelSolver;
    btDiscreteDynamicsWorld* dynamicsWorld;

    // avatar
    float m_fRadius = 0.28f;
    float m_fHeight = 0.64f;
    btRigidBody *rigidbodyAvatar;

    // boxes
    Model m_modelBox;
    QList < btRigidBody* > m_listRigidBodies;
    QList<GLuint> m_listRigidBodiesTextureId;

    int numTextures = 6;
    QString strFilenames[6] = {"diffus_black.tga", "diffus_blue.tga", "diffus_green.tga", "diffus_red.tga", "diffus_rust.tga", "diffus_yellow.tga"};
    GLuint textures[6];
};
#endif // MAINWINDOW_H
